Package: autotune
Type: Package
Title: Fast, Accurate and Automatic Tuning Parameter Selection for
        Lasso
Version: 0.1.1
Maintainer: Tathagata Sadhukhan <ts767@cornell.edu>
Description: Fits Lasso paths for high-dimensional regression using coordinate descent with automatic, data-driven tuning of the regularization parameter. The implementation is 10 to 200 times faster than the standard 'glmnet' implementation of Lasso tuned via Cross Validation and over 100 times faster than scaled Lasso. It also provides a reliable estimate of the regression noise level and a new diagnostic for sparsity.
    For details of the method, see Sadhukhan, Wilms, Smeekes and Basu (2025)
    "Autotune: fast, accurate, and automatic tuning parameter selection for Lasso"
    <doi:10.48550/arXiv.2512.11139>.
License: GPL (>= 2)
Encoding: UTF-8
Depends: R (>= 2.10)
Imports: Rcpp (>= 1.0.13)
LinkingTo: Rcpp
RoxygenNote: 7.3.2
Suggests: knitr, rmarkdown, glmnet, AUC, ggplot2, ggExtra, dplyr,
        tidyr, Matrix
VignetteBuilder: knitr
Authors@R: c(person(given = "Tathagata",
                        family = "Sadhukhan",
                        role = c("aut", "cre"),
                        email = "ts767@cornell.edu"),
                 person(given = "Ines",
                        family = "Wilms",
                        role = "aut"),
                 person(given = "Stephan",
                        family = "Smeekes",
                        role = "aut"),
                 person(given = "Sumanta",
                        family = "Basu",
                        role = "aut"))
NeedsCompilation: yes
Packaged: 2026-09-02 15:32:23 UTC; ts767
Author: Tathagata Sadhukhan [aut, cre],
  Ines Wilms [aut],
  Stephan Smeekes [aut],
  Sumanta Basu [aut]
Repository: CRAN
Date/Publication: 2026-09-02 16:20:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-09-02 17:00:24 UTC; unix
Archs: autotune.so.dSYM

S3method(plot,autotune_lasso)
S3method(coef,autotune_lasso)
S3method(coef,autotune_lasso_path)
S3method(predict,autotune_lasso)
export(autotune_lasso)
importFrom(Rcpp, evalCpp)
useDynLib(autotune, .registration = TRUE)
importFrom("graphics", "abline", "grid", "legend", "lines", "par", "points", "text")
importFrom("stats", "coef", "lm")
importFrom("utils", "tail")

# autotune
Repository for R package of autotune Lasso

## Installation
You can install the development version from GitHub:

```r
# Install the devtools or remotes package if you don't have it
# install.packages("devtools")

# Ensure that you have the Rcpp package installed with version >=1.0.13
devtools::install_github("Tathagata-S/autotune")
```
When installing from GitHub, in order to build the package from source, you need to have the appropriate R development tools installed ([Rtools](https://cran.r-project.org/bin/windows/Rtools/) on Windows, or [these tools](https://mac.r-project.org/tools/) on Mac).


## Installation with vignette
You can also the build the vignette while installing the development version from GitHub:

```r
# install.packages("devtools")

devtools::install_github("Tathagata-S/autotune", build_vignettes = TRUE, dependencies = TRUE)

vignette("autotune-lasso-vignette")
```

👉 **Usage**:
```md
Here’s a quick example:
```

```r
library(autotune)
?autotune_lasso

set.seed(10)
n <- 80
p <- 400
s <- 5
snr <- 4
betatrue <- c(rep(1,s), rep(0, p - s))
x <- matrix(rnorm(n * p), ncol = p)
error.sd <- sqrt((betatrue %*% betatrue)/snr)

err <- rnorm(n, sd = error.sd)
y <- x %*% betatrue + err
y <- y - mean(y)

ans <- autotune_lasso(x, y, trace_it = T)

b <- betatrue
# The Predictors which are actually significant:
which(b != 0)
# The Predictors which had nonzero estimated coefficients:
which(ans$beta != 0)
# Top 10 predictors X_i's in the ranking of X_i's given by autotune:
ans$CD.path.details$sorted_predictors[1:10]
# No of significant predictors in each CD iteration when sigma_hat is allowed to vary:
ans$CD.path.details$count_sig_beta
# Sigma estimates in each CD iteration:
ans$CD.path.details$sigma_sq_seq
# Empirical noise variance:
var(err)
```

#' Extract coefficients for autotune_lasso objects
#'
#' This function extracts the regression coefficients estimated by an autotune
#' lasso model, using the stored \code{"autotune_lasso"} object
#'
#'
#' @param object An \code{"autotune_lasso"} object (for \code{coef.autotune_lasso})
#'   or \code{"autotune_lasso_path"} object (for \code{coef.autotune_lasso_path}).
#' @param intercept Logical; include the intercept term? If \code{FALSE}, the intercept
#'   row is omitted.
#' @param path Logical; if \code{TRUE}, return the entire coefficient path across
#' different lambdas.
#' @param sparse Logical; controls the output format of \code{coef.autotune_lasso()} when
#'   \code{path = FALSE}. If \code{TRUE}, returns a sparse \code{dgCMatrix} (glmnet-style);
#'   if \code{FALSE}, returns a base numeric vector.
#' @param a0 Intercept of the fitted autotune regression. Used only when \code{intercept = TRUE}.
#' @param ... Other arguments to coef.
#'
#' @return
#' \itemize{
#'   \item \code{coef.autotune_lasso(path = FALSE, sparse = TRUE)} returns a sparse \code{dgCMatrix}
#'     of dimension (\code{nvars} + 1) \eqn{\times 1} (or \code{nvars}  \eqn{\times 1} if \code{intercept = FALSE}).
#'   \item \code{coef.autotune_lasso(path = FALSE, sparse = FALSE)} returns a named numeric vector.
#'   \item \code{coef.autotune_lasso(path = TRUE)} calls \code{coef.autotune_lasso_path()}, which returns a sparse
#'     \code{dgCMatrix} of dimension (\code{nvars}  + 1) \eqn{\times} (k + 1) (or \code{nvars}  \eqn{\times} (k + 1) if
#'     \code{intercept = FALSE}), with columns named \code{lambda_1, ..., lambda_k, final_lambda}.
#' }
#'
#' @examples
#' set.seed(10)
#' n = 300
#' p = 500
#' s = 10
#' beta = c(rep(1, s), rep(0, p - s))
#' x = matrix(rnorm(n * p), ncol = p)
#' # Maunal sigma allocation
#' # y = x %*% beta + rnorm(n, sd = 1)
#' # Dynamic sigma allocation with snr specified
#' snr = 2
#' y = x %*% beta + rnorm(n, sd = sqrt(var(x%*%beta)/snr))
#' fit <- autotune_lasso(x, y)
#' coef(fit)                    # final solution
#' coef(fit, path = TRUE)        # coefficient path (sparse)
#'
#' @importFrom Matrix sparseMatrix Matrix t rBind
#' @name coef.autotune_lasso
NULL

#' Data-adaptive automatic and fast tuning of LM with Lasso regularization
#' @name autotune_lasso
#' @param x Matrix of predictors, with one column for each predictor.
#' Dimension will be \code{nobs} \eqn{\times} \code{nvars}; so each row is a new observation
#' and \code{nvars} > 1.
#' @param y Vector of responses.
#' @param alpha Default 0.01, significance level of sequential F-tests 
#' used for estimation of support set.
#' @param standardize Logical flag for standardization of all variables in x, prior to
#' fitting the model sequence. The coefficients are always returned on the
#' original scale. Default value is \code{TRUE}.
#' @param standardize_response Logical flag for demeaning the reponse variable y.
#' Default value is \code{TRUE}.
#' @param intercept Should intercept(s) be fitted (default=\code{TRUE}) or set to zero
#' (\code{FALSE}).
#' @param active Should active set selection be used (\code{TRUE}) or avoided
#' (default = \code{FALSE}). It is under experimentation phase, use it 
#' with care.
#' @param trace_it Logical input, default \code{FALSE}; if \code{TRUE}
#' prints out the iteration number while running, useful for big datasets that 
#' take a long time to fit.
#' @param tolerance Numeric input for an additional stopping criteria on the
#' coordinate descent when sigma is updating. Stops that coordinate
#' descent when the relative change between successive iterates of
#' coefficients' estimates is less than the \code{tolerance}. Default value 
#' is \eqn{10^{-4}}.
#' @param beta_tolerance Numeric input for the stopping criteria on the
#' coordinate descent when sigma is not updating. Stops that coordinate
#' descent when the relative change between successive iterates of
#' coefficients' estimates is less than the \code{beta_tolerance}.
#' Default value is \eqn{10^{-3}}.
#' @param iter_max Maximum number of iterations of coordinate descent
#' allowed when sigma is updating. Default value is 30.
#' @param beta_iter_max Maximum number of iterations of coordinate 
#' descent allowed when sigma is not updating. Default value is 40.
#' @param active_iter_max If \code{active = TRUE}, maximum number of 
#' times the active set is updated as per the violations of KKT 
#' conditions. Default value is 5.
#' @param PR_norm_l2 Logical flag to whether use the l2 norm of partial 
#' residuals for ordering them instead of the default l1 norm. 
#' @param ... Additional arguments passed to the internal fitting routine.
#' 
#' 
#' @return A list with various intermediate and final outputs produced by autotune Lasso in its regularization path.
#' 
#' \item{beta}{ Final estimates of regression coefficients. }
#' \item{a0}{ Intercept of the fit. }
#' \item{lambda}{Final thresholding value \eqn{\lambda =
#' \lambda_0\hat\sigma^2} used in the coordinate descent after noise
#' variance estimate \eqn{\hat\sigma^2} has converged.}
#' \item{sigma_sq}{ Final estimate of noise variance \eqn{\sigma^2}}
#' \item{nobs}{ Number of observations.}
#' \item{nvars}{ Number of variables.}
#' \item{CD.path.details}{ A list of additional details about the coordinate descent path taken by
#' autotune Lasso:
#'   \describe{
#'     \item{\code{sorted_predictors}}{Decreasing ordering of predictors in terms of
#'     their contribution to predicting the response values.}
#'     \item{\code{sigma_sq_seq}}{A \code{no_of_iterations}-length sequence of
#'     noise variance estimates \eqn{\sigma^2}.}
#'     \item{\code{beta_matrix}}{A (\code{length(sigma_sq_seq) + 1})
#'     \eqn{\times} \code{nvars} matrix of estimated coefficients.}
#'     \item{\code{no_of_iter_before_lambda_conv}}{Number of coordinate descent
#'     iterations performed before the noise variance estimate
#'     \eqn{\hat{\sigma}^2} converged.}
#'     \item{\code{no_of_iter_after_lambda_conv}}{After the noise variance estimate
#'     \eqn{\hat{\sigma}^2} has converged, the number of coordinate descent
#'     iterations required for coefficients \eqn{\hat\beta} to converge.}
#'     \item{\code{no_of_iterations}}{Total number of coordinate descent iterations
#'     implemented by autotune Lasso.}
#'     \item{\code{lambda0}}{Value of \eqn{\lambda_0} used in autotune Lasso.
#'     Refer to the original paper for details.}
#'     \item{\code{support_set}}{Final set of predictors included in the support set
#'     for noise variance estimation by autotune Lasso.}
#'     \item{\code{count_sig_beta}}{A \code{no_of_iterations}-length vector containing
#'     the support set sizes across the coordinate descent iterations while the
#'     noise variance estimate is being updated.}
#'     \item{\code{null_support}}{Boolean output indicating whether the final support
#'     set estimate is a null set. If so, \code{autotune_lasso} uses the support set
#'     estimate from the previous iteration to obtain the final noise variance
#'     estimate \eqn{\sigma^2}.}
#'     \item{\code{active_iterations}}{If \code{active = TRUE}, the number of times
#'     the active set is updated according to violations of the KKT conditions.}
#'     \item{\code{active_set_sizes}}{An \code{active_iterations}-length vector
#'     containing the active-set sizes used for coordinate descent across the
#'     active-set iterations.}
#'   }
#' }
#' 
#' 
#' 
#' 
#' @description
#' Fits a linear model via alternative optimization of penalized gaussian maximum likelihood 
#' which is a biconvex function of regression coefficients \eqn{\beta} and noise variance \eqn{\sigma^2}. 
#' The regularization path of \code{autotune_lasso} quickly picks out a good lambda for Lasso and then
#' returns the corresponding linear fit along with various attributes related to the fit.
#' 
#' 
#' @export
#'
#' @examples
#' library(autotune)
#' set.seed(10)
#' n <- 80
#' p <- 400
#' s <- 5
#' snr <- 4
#' betatrue <- c(rep(1,s), rep(0, p - s))
#' x <- matrix(rnorm(n * p), ncol = p)
#' error.sd <- sqrt((betatrue %*% betatrue)/snr)
#' err <- rnorm(n, sd = error.sd)
#' y <- x %*% betatrue + err
#' ans <- autotune_lasso(x, y, trace_it = TRUE)
#' b <- betatrue
#' # The Predictors which are actually significant:
#' which(b != 0)
#' # The Predictors which had nonzero estmated coefficients:
#' which(ans$beta != 0)
#' # Top 10 predictors X_i's in the ranking of X_i's given by autotune:
#' ans$CD.path.details$sorted_predictors[1:10]
#' # Cardinality of autotune's estimated support set in each CD iteration before lambda converged:
#' ans$CD.path.details$count_sig_beta
#' # Sigma estimates in each CD iteration:
#' ans$CD.path.details$sigma_sq_seq
#' # Empirical noise variance:
#' var(err)
#' 
#' 
autotune_lasso <- function(x,
                           y,
                           alpha = 0.01,
                           standardize = TRUE,
                           standardize_response = TRUE,
                           intercept = TRUE,
                           active = FALSE,
                           trace_it = FALSE,
                           tolerance = 1e-4,
                           beta_tolerance = 1e-3,
                           iter_max = 30,
                           beta_iter_max = 40,
                           active_iter_max = 5,
                           PR_norm_l2 = FALSE,
                           ...) {
  cl <- match.call()
  
  fit <- autotune_lasso_cpp(
    xin = x,
    yin = y,
    alpha = alpha,
    standardize = standardize,
    standardize_response = standardize_response,
    intercept = intercept,
    active = active,
    trace_it = trace_it,
    tolerance = tolerance,
    beta_tolerance = beta_tolerance,
    iter_max = iter_max,
    beta_iter_max = beta_iter_max,
    active_iter_max = active_iter_max,
    PR_norm_l2 = PR_norm_l2,
    ...
  )
  fit$call <- cl
  fit$x <- x
  fit$y <- y
  fit
}

#' autotune: Lasso with data-driven tuning for linear models
#'
#' This package fits lasso path for high-dimensional regression using
#' coordinate descent while automatically tuning it's regularization parameter.
#' autotune Lasso is 10-50 times than the standard glmnet implementation of lasso,
#' and over 100 times faster than scaled lasso. Additionally, it gives reliable 
#' noise level estimate for the regression problem. 
#'
#' \tabular{ll}{ Package: \tab autotune\cr Type: \tab Package\cr Version: \tab
#' 1.0\cr Date: \tab 2025-09-18\cr }
#' Very simple to use. Accepts \code{x,y} data for linear regression model,
#' and produces the regularization path which automatically estimates an
#' optimal tuning parameter \code{lambda}.\cr
#' @section Functions: Only 1 function, more autotune algorithms for VAR, random forests,
#' coming soon!
#'   \itemize{
#'     \item \code{\link{autotune_lasso}}
#'   }
#' 
#' @author Tathagata Sadhukhan Ines Wilms Stephan Smeekes
#' and Sumanta Basu\cr Maintainer: Tathagata Sadhukhan(ts767@@cornell.edu)
#' @docType package
#' @name autotune-package
#' @keywords lasso data-driven-tuning package fast accurate 
#' 
#' @examples
#' 
#' set.seed(10)
#' n <- 80
#' p <- 500
#' s <- 5
#' type <- 1
#' snr <- 3
#' betatrue <- c(rep(1,s), rep(0, p - s))
#' x <- scale(matrix(rnorm(n * p), ncol = p))
#' error.sd <- sqrt((betatrue %*% betatrue)/snr)
#' err <- rnorm(n, sd = error.sd)
#' y <- x %*% betatrue + err
#' y <- y - mean(y)
#' ans <- autotune_lasso(x, y, trace_it = TRUE)
#' b <- betatrue
#' # The Predictors which are actually significant:
#' which(b != 0)
#' # The Predictors which had nonzero estmated coefficients:
#' which(ans$beta != 0)
#' # Top 10 predictors X_i's in the ranking of X_i's given by autotune:
#' ans$sorted_predictors[1:10] + 1
#' # No of significant predictors in each CD iteration when sigma_hat is allowed to vary:
#' ans$count_sig_beta
#' # Sigma estimates in each CD iteration:
#' ans$sigma2_seq
#' # Empirical noise variance:
#' var(err)
#' 
#' 
"_PACKAGE"

#' @rdname coef.autotune_lasso
#' @method coef autotune_lasso
#' @export

coef.autotune_lasso <- function(object, intercept = TRUE, path = FALSE, sparse = TRUE, ...) {
  
  if (!is.logical(path) || length(path) != 1L || is.na(path)) {
    stop("'path' must be a single TRUE or FALSE value.")
  }

  if (path) {
    return(coef(object$CD.path.details,
                intercept = intercept,
                a0 = object$a0))
  }
  
  if (sparse && !requireNamespace("Matrix", quietly = TRUE)) {
    stop("Package 'Matrix' must be installed to return sparse coefficients.")
  }
  
  beta_hat <- c(object$a0, object$beta)
  varnames <- c("(Intercept)", paste0("V", seq_len(object$nvars)))
  names(beta_hat) <- varnames
  
  if (!intercept) {
    beta_hat <- beta_hat[-1]
    varnames <- varnames[-1]
  }
  
  if (!sparse) return(beta_hat)
  
  nz <- which(beta_hat != 0)
  
  sparse_beta <- Matrix::sparseMatrix(
    i = nz,
    j = rep.int(1L, length(nz)),
    x = beta_hat[nz],
    dims = c(length(beta_hat), 1L),
    dimnames = list(varnames, NULL)
  )
  
  sparse_beta
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----install-github, eval = FALSE---------------------------------------------
# # install.packages("devtools")
# # Ensure that you have the Rcpp package installed with version >=1.0.13
# # devtools::install_github("Tathagata-S/autotune")

## ----setup--------------------------------------------------------------------
library(autotune)

## -----------------------------------------------------------------------------
set.seed(10)
n = 300
p = 500
s = 10
beta = c(rep(1, s), rep(0, p - s))
x = matrix(rnorm(n * p), ncol = p)
# Maunal sigma allocation
# y = x %*% beta + rnorm(n, sd = 1)
# Dynamic sigma allocation with snr specified
snr = 2
y = x %*% beta + rnorm(n, sd = sqrt(var(x%*%beta)/snr))

## -----------------------------------------------------------------------------
ptm <- proc.time()
fit.autotune <- autotune_lasso(x, y, alpha = 0.01)
proc.time() - ptm

## -----------------------------------------------------------------------------
b.autotune <- fit.autotune$beta
intercept.autotune <- fit.autotune$a0
lambda.autotune <- fit.autotune$lambda
sigma.seq.autotune <- fit.autotune$CD.path.details$sigma_sq_seq
sigma.estimate.autotune <- fit.autotune$sigma_sq

## -----------------------------------------------------------------------------
head(coef(fit.autotune), n = 21)

## -----------------------------------------------------------------------------
library(glmnet)

## -----------------------------------------------------------------------------
ptm2 <- proc.time()
fit.glmnet <- cv.glmnet(x, y)
proc.time()-ptm2

## ----fig.width=7.5, fig.height=7.5--------------------------------------------
plot(fit.glmnet, cex.lab = 1.5, ylab = "10-fold CV MSE")

# plotting log of lambdas selected by CV
abline(v = log(c(fit.glmnet$lambda.min, fit.glmnet$lambda.1se)), lty = "dashed",col = c(rgb(0,1,0), rgb(0,0.75,0)), lwd = 3)

# plotting log of lambda selected by autotune
abline(v = log(lambda.autotune), col = "blue", lty = "dashed", lwd = 3)
legend("topleft", inset = 0.05,             legend = c("Autotune LASSO",
             "CV_LASSO(min)",
             "CV_LASSO(1se)"),
  col = c("blue", rgb(0,1,0), rgb(0,0.75,0)),
  lty = "dashed", lwd = 2, cex = 1)

## ----fig.width=7.5, fig.height=7.5--------------------------------------------
mse_glmnet <- apply(fit.glmnet$glmnet.fit$beta, 2, function(x) mean((x- beta)^2)/mean(beta^2))
ymin = 0.95 * min(mse_glmnet)
ymax = max(1, max(mse_glmnet))
oldpar <- par(mgp = c(3.5, 1, 0), mar = c(5, 4, 2.5, 0.5) + 1.4)
plot(log(fit.glmnet$lambda), mse_glmnet, 
     type = 'b', col = "red", cex.lab = 2, cex=2, cex.axis = 2,
     ylim = c(ymin, ymax),
     ylab = "Relative MSE w.r.t. true coefficients", xlab = expression(paste("log(",lambda,")")))

# plotting log of lambdas selected by CV
abline(v = log(c(fit.glmnet$lambda.min, fit.glmnet$lambda.1se)), lty = "dashed",col = c(rgb(0,1,0), rgb(0,0.75,0)), lwd = 3)

# plotting log of lambdas selected by autotune
abline(v = log(lambda.autotune), col = "blue", lty = "dashed", lwd = 3)
legend(
  "topleft", inset = 0.05,                  
  legend = c("Autotune LASSO",
             "CV_LASSO(min)",
             "CV_LASSO(1se)"),
  col = c("blue", rgb(0,1,0), rgb(0,0.75,0)),
  lty = "dashed", lwd = 2, cex = 1)
par(oldpar)

## ----fig.width=7.5, fig.height=7.5--------------------------------------------
temp <- fit.autotune$CD.path.details$lambda0
seq.lambdas.autotune <- temp * c(var(y), sigma.seq.autotune)

final.rmse.autotune <- (mean(b.autotune - beta)^2) / (mean(beta)^2)
intermediate.rmse.autotune <- approx(x = fit.glmnet$lambda, y = mse_glmnet, xout = seq.lambdas.autotune)
intermediate.rmse.autotune$y[1] <- 1
rmse.path.autotune <- c(intermediate.rmse.autotune$y, final.rmse.autotune)

oldpar <- par(mgp = c(3.5, 1, 0), mar = c(5, 4, 2.5, 0.5) + 1.4)
plot(log(fit.glmnet$lambda), mse_glmnet, 
     type = 'b', col = "red", cex.lab = 2, cex=2, cex.axis = 2,
     ylim = c(min(ymin, 0.95 * final.rmse.autotune), ymax),
     ylab = "Relative MSE w.r.t. the true coefficients", xlab = expression(paste("log(",lambda,")")))

lines(c(log(seq.lambdas.autotune), log(fit.autotune$lambda)), 
      rmse.path.autotune, 
      col = "blue", 
      lwd = 2)                  
points(log(seq.lambdas.autotune), 
       intermediate.rmse.autotune$y, 
       col = "blue", 
       pch = 1,               
       lwd = 2,
       cex = 2) 
points(log(fit.autotune$lambda),
      final.rmse.autotune,
      col = "blue",
      pch = 11,
      lwd = 3,
      cex = 3
    )
par(oldpar)

## -----------------------------------------------------------------------------
library(AUC)

## -----------------------------------------------------------------------------
b.glmnet = coef(fit.glmnet, s = "lambda.min")[-1]

## -----------------------------------------------------------------------------
auc(roc(abs(b.autotune), as.factor(beta != 0)))
auc(roc(abs(b.glmnet), as.factor(beta != 0)))

## -----------------------------------------------------------------------------
data("sp500")
attach(sp500)

scaled_lasso_benchmark <- read.csv(
  system.file(
    "extdata",
    "sp500-scalreg-1.0.1.csv",
    package = "autotune"
  )
)
scaled_lasso_observed <- scaled_lasso_benchmark[
  which(!scaled_lasso_benchmark$bootstrap),
  ,
  drop = FALSE
]

n <- 190
rang <- 252 - n

X_train = sp500.percent[1:n, 3:(dim(sp500.percent)[2])]
Y_train = sp500.percent[1:n, 1]

x_train = scale(X_train)
y_train = Y_train - mean(Y_train)

X_test = sp500.percent[(1:rang) + n, 3: (dim(sp500.percent)[2])]
Y_test = sp500.percent[(1:rang) + n, 1]
x_test <- scale(X_test)
y_test <- Y_test - mean(Y_test)
# Default value of beta_iter_max is 40
ans_autotune <- autotune_lasso(x_train, y_train, beta_iter_max = 40, trace_it = TRUE)
pred_err_autotune <- mean( (y_test - x_test %*% ans_autotune$beta)^2 )

pred_err_scallas <- scaled_lasso_observed$test_mse

cv_fit <- cv.glmnet(x_train, y_train, alpha = 1, intercept = F)
pred_err_cv_min <- mean( (y_test - predict(cv_fit, newx = x_test, s = "lambda.min"))^2 )
pred_err_cv_1se <- mean( (y_test - predict(cv_fit, newx = x_test, s = "lambda.1se"))^2 )


pred_err_autotune
pred_err_scallas
pred_err_cv_min
pred_err_cv_1se

scaled_lasso_observed$nonzero_coefs
sum(ans_autotune$beta != 0)
sum(coef(cv_fit, s = "lambda.min") != 0)
sum(coef(cv_fit, s = "lambda.1se") != 0)
detach(sp500)

## -----------------------------------------------------------------------------
real_life_prediction_comparison <- function(x, y, x_test, y_test, scaled_lasso_result, alpha = 0.01, beta_iter_max = 40, plot_regu = TRUE, trace_it = TRUE){

ans_autotune <- autotune_lasso(x, y, alpha = alpha, beta_iter_max = beta_iter_max, trace_it = trace_it)
# ans_glmnet <- glmnet(x, y, alpha = 1, intercept = FALSE)
cv_fit <- cv.glmnet(x, y, alpha = 1, intercept = F)
vary <- var(y_test)
mse_glmnet <- apply(cv_fit$glmnet.fit$beta, 2, function(b) mean((y_test - x_test%*%b)^2)/vary)

# log_glmnet_lambdas <- log(fit.glmnet$lambda)
# log_autotune_lambdas <- log(intermediate_lambdas_used)



if(plot_regu) {
  intermediate_lambdas_used <- c( c(var(y)/2, ans_autotune$CD.path.details$sigma_sq_seq) * ans_autotune$CD.path.details$lambda0)
  
  autotune_lambdas <- c(intermediate_lambdas_used, ans_autotune$lambda)
  
  final.rmse.autotune <- mean((y_test - x_test%*%ans_autotune$beta)^2) / vary
  intermediate.rmse.autotune <- approx(x = log(cv_fit$lambda), y = mse_glmnet, xout = log(intermediate_lambdas_used), rule = 2)
  intermediate.rmse.autotune$y[1] <- 1
  rmse.path.autotune <- c(intermediate.rmse.autotune$y, final.rmse.autotune)
  
  
  ysquaredmean <- mean(y^2)
  cv_fit$cvm <- cv_fit$cvm/ysquaredmean
  cv_fit$cvsd <- cv_fit$cvsd/ysquaredmean
  cv_fit$cvup <- cv_fit$cvm + cv_fit$cvsd
  cv_fit$cvlo <- cv_fit$cvm - cv_fit$cvsd
  
  cv_fit_lowerlim <- min(cv_fit$cvm - cv_fit$cvsd, mse_glmnet)
  cv_fit_upperlim <- max(cv_fit$cvm + cv_fit$cvsd, mse_glmnet)
  
  ymin = 0.95 * min(cv_fit_lowerlim , min(rmse.path.autotune))
  ymax = max(cv_fit_upperlim, rmse.path.autotune)
  ymax_trunc = ymin + 0.8 * (ymax - ymin)  # Truncate vertical lines here
  
  ydiff = ymax - ymin
  oldpar <- par(mgp = c(3.5, 1, 0), mar = c(6, 5, 4, 2) + 0.1)
  on.exit(par(oldpar), add = TRUE)
  
  par(mar = c(5, 7, 4, 2))
  plot(cv_fit,
       ylim = c(ymin, max(cv_fit_upperlim, rmse.path.autotune)),
       cex.lab = 2,
       cex.axis = 2,
       ylab = expression(
    "||" * (Y[test] - X[test] * hat(beta)) * "||"[2]^2
    ~ "/" ~
    "||" * Y[test]  * "||"[2]^2
  ))
  # expression(paste(sqrt("RMSE")))
  
  points(log(cv_fit$lambda), mse_glmnet,
         type = 'b', col = "orange", lwd = 3)
  
  
  lines(rep(log(ans_autotune$lambda), 2),
        c(ymin, ymin + 0.92 * ydiff), col = "blue", lwd = 0.5, lty = "dashed")
  
  lines(log(autotune_lambdas), rmse.path.autotune,
        col = "blue", lwd = 1.5)
  
  points(log(autotune_lambdas), rmse.path.autotune,
         col = "blue", pch = 1, lwd = 1.5, cex = 1)
  
  points(log(ans_autotune$lambda), final.rmse.autotune,
         col = "blue", pch = 11, lwd = 2, cex = 2)
  
  
  
  vlines <- c(
    log(cv_fit$lambda.min),
    log(cv_fit$lambda.1se)
  )
  vline_labels <- c("CV(min)", "CV(1se)")
  vline_colors <- c(rgb(0,1,0), rgb(0,0.75,0))
  vline_ltys <- c("dashed", "dashed")
  vline_lwds <- c(3, 3)
  
  for (i in seq_along(vlines)) {
    lines(rep(vlines[i], 2),
          c(ymin, ymin + (0.55 + i * 0.1) * ydiff),
          col = vline_colors[i], lty = vline_ltys[i],
          lwd = vline_lwds[i])
  
    text(x = vlines[i], y = ymin + (0.57 + i * 0.1) * ydiff,
         labels = vline_labels[i], srt = -30, adj = 0,
         xpd = TRUE, cex = 1.4, col = vline_colors[i])
  }
  
  text(x = log(ans_autotune$lambda),
       y =  ymin + 0.95 * ydiff,
       labels = "Autotune", srt = -30, adj = 0,
       xpd = TRUE, cex = 1.4, col = "blue")
  
  if (!is.null(cv_fit$cvsd)) {
    arrows(x0 = log(cv_fit$lambda), 
           y0 = cv_fit$cvm - cv_fit$cvsd,
           x1 = log(cv_fit$lambda),
           y1 = cv_fit$cvm + cv_fit$cvsd,
           angle = 90, code = 3, length = 0.05, col = "red", lwd = 1.5)
  }
  
  
  legend("topright",
         inset = c(0.05, 0.02),
         legend = c("CV Error", "Test Error"),
         col = c("red", "orange"),
         pt.cex = c(1.5, 2), pch = c(19, 19),
         lwd = c(1, 3),
         text.col = c("red", "orange"),
         horiz = FALSE,
         bty = "n")

}

# mse_autotune <- mean( (y_test - x_test %*% ans_autotune$beta)^2 )
# mse_cv_min <- mean( (y_test - predict(cv_fit, newx = x_test, s = "lambda.min"))^2 )
# mse_cv_1se <- mean( (y_test - predict(cv_fit, newx = x_test, s = "lambda.1se"))^2 )


oos_scallas <- scaled_lasso_result$out_of_sample_r2
oos_autotune <- 1 -  mean( (y_test - x_test %*% ans_autotune$beta)^2 ) / vary
oos_cv_min <- 1 - mean( (y_test - predict(cv_fit, newx = x_test, s = "lambda.min"))^2 ) / vary
oos_cv_1se <- 1 - mean( (y_test - predict(cv_fit, newx = x_test, s = "lambda.1se"))^2 ) / vary

return(data.frame(out_of_sample_autotune = oos_autotune,
out_of_sample_scaled_lasso = oos_scallas,
out_of_sample_cv_min = oos_cv_min,
out_of_sample_cv_1se = oos_cv_1se,
nonzero_coefs_autotune = sum(ans_autotune$beta != 0),
nonzero_coefs_scaled_lasso = scaled_lasso_result$nonzero_coefs,
nonzero_coefs_cvmin = sum(coef(cv_fit, s = "lambda.min") != 0),
nonzero_coefs_cv1se = sum(coef(cv_fit, s = "lambda.1se") != 0)))
}

## ----fig.width=7.5, fig.height=7.5--------------------------------------------
ans <- real_life_prediction_comparison(
  x_train,
  y_train,
  x_test,
  y_test,
  scaled_lasso_result = scaled_lasso_observed,
  plot_regu = TRUE
)
names(ans)

## -----------------------------------------------------------------------------
library(dplyr)

## ----fig.width=7.5, fig.height=7.5--------------------------------------------
#Number of bootstraps
B <- 20


result_list <- list()

result_list[[1]] <- data.frame(
  n = n,
  bootstrap = FALSE,
  real_life_prediction_comparison(
    x_train,
    y_train,
    x_test,
    y_test,
    scaled_lasso_result = scaled_lasso_observed,
    plot_regu = FALSE,
    trace_it = FALSE
  )[1:4]
)

for( j in (1:B)+1) {
  set.seed(j)
  boot.sample <- sample(n, size = n, replace = TRUE)
  boot.x <- x_train[boot.sample, ]
  boot.y <- y_train[boot.sample]
  scaled_lasso_result <- scaled_lasso_benchmark[
    which(scaled_lasso_benchmark$seed == j),
    ,
    drop = FALSE
  ]
  result_list[[j]] <- c(
    n = n,
    bootstrap = TRUE,
    real_life_prediction_comparison(
      boot.x,
      boot.y,
      x_test,
      y_test,
      scaled_lasso_result = scaled_lasso_result,
      plot_regu = FALSE,
      trace_it = FALSE
    )[1:4]
  )
}

mat <- do.call(rbind, result_list)     
df <- as.data.frame(mat, stringsAsFactors = FALSE)

colnames(df) <- c("n", "bootstrap", "Autotune",
                  "Scaled", "CVmin",
                  "CV1se")
df <- df %>%
  mutate(
    n = as.integer(n),
    bootstrap = as.logical(bootstrap), 
    across(!c(n, bootstrap), as.numeric)
  )

## ----fig.width=7.5, fig.height=7.5--------------------------------------------
alg_cols <- colnames(df)[3:6]
boot_only <- df %>% filter(bootstrap == TRUE)
boxlist <- lapply(alg_cols, function(cn) boot_only[[cn]])
names(boxlist) <- alg_cols

# Uncomment the following line and line 448 for saving the boxplot in your working directory

# pdf(file.path(paste0("OOS_boxplot_for_n=",df[1,1],"_B=", B,".pdf")), width = 8, height = 9)

oldpar <- par(mar = c(4, 6, 4, 2))
boxplot(boxlist, main = paste0("Bootstrapped Out-of-Sample R^2 for n = ", df[1,1]), ylab = expression(paste("Out-of-Sample ",R^2)), ylim = c(max(0.98 * min(df[alg_cols]), 0), 1),
        cex.lab = 2,     
  cex.axis = 1.8,    
  cex.main = 1.5)

nonboot <- df %>% filter(bootstrap == FALSE)
if(nrow(nonboot) > 0){
  points(1:4, as.numeric(nonboot[1, alg_cols]), pch = 18, col = "red", cex = 2)
}

legend(
  "topright",
  legend = expression(paste("Observed O.O.S. ",R^2)),
  col = "red",
  pch = 18,
  pt.cex = 2,
  cex = 2,          
  bty = "n"           
)

par(oldpar)
# dev.off()

#' @rdname coef.autotune_lasso
#' @method coef autotune_lasso_path
#' @export
coef.autotune_lasso_path <- function(object,
                                intercept = TRUE,
                                a0 = NULL, ...) {
  
  if (!requireNamespace("Matrix", quietly = TRUE)) {
    stop("Package 'Matrix' must be installed to return sparse coefficients.")
  }
  
  beta_mat <- object[["beta_matrix"]]  # (k+1) x p
  if (is.null(beta_mat)) {
    stop("'object' does not contain a 'beta_matrix' component.")
  }
  if (!is.matrix(beta_mat)) beta_mat <- as.matrix(beta_mat)
  
  beta_sp <- Matrix::Matrix(t(beta_mat), sparse = TRUE)  # p x (k+1)
  
  n_solutions <- ncol(beta_sp)
  k <- n_solutions - 1L
  p <- nrow(beta_sp)
  
  soln_names <- if (k > 0L) {
    c(paste0("lambda_", seq_len(k)), "final_lambda")
  } else {
    "final_lambda"
  }
  colnames(beta_sp) <- soln_names
  varnames <- colnames(beta_mat)
  if (is.null(varnames)) varnames <- paste0("V", seq_len(p))
  rownames(beta_sp) <- varnames
  if (intercept) {
    if (is.null(a0)) {
      stop("'a0' must be supplied when 'intercept = TRUE'.")
    }
    if (length(a0) == 1L) a0 <- rep(a0, n_solutions)
    if (length(a0) != n_solutions) {
      stop("'a0' must have length 1 or the number of path solutions.")
    }
    beta_sp <- rbind(a0, beta_sp)
    rownames(beta_sp) <- c("(Intercept)", varnames)
  }
  beta_sp
}

#' Predict from a autotune_lasso fit
#'
#' @param object An object of class \code{autotune_lasso}.
#' @param newx Numeric matrix of new \code{x} (rows = observations,
#'   columns = predictors) at which predictions are required. This 
#'   argument is not used for \code{type="coefficients")}.
#' @param type Character; one of \code{"response"}, \code{"coefficients"}.
#' @param path For \code{type = "coefficients"}, \code{NULL} returns the final
#'   coefficients and \code{TRUE} returns the full coefficient path. This
#'   argument is ignored for \code{type = "response"}.
#' @param ... Further arguments passed from the generic.
#'
#' @return
#' If \code{type = "response"}, returns the prediction at the x provided for prediction.
#'
#' If \code{type = "coefficients"}, returns the fitted coefficients.
#'
#'
#' @examples
#' set.seed(10)
#' n = 300
#' p = 500
#' s = 10
#' beta = c(rep(1, s), rep(0, p - s))
#' x = matrix(rnorm(n * p), ncol = p)
#' # Maunal sigma allocation
#' # y = x %*% beta + rnorm(n, sd = 1)
#' # Dynamic sigma allocation with snr specified
#' snr = 2
#' y = x %*% beta + rnorm(n, sd = sqrt(var(x%*%beta)/snr))
#' fit <- autotune_lasso(x, y)
#' predict(fit, newx = x[1:5, , drop = FALSE])
#' predict(fit, type = "coefficients")
#' 
#' predict(fit, type = "coefficients", path = TRUE)
#'
#' @method predict autotune_lasso
#' @export
predict.autotune_lasso <- function(object,
                              newx = NULL,
                              type = c("response", "coefficients"),
                              path = NULL,
                              ...) {
  type <- match.arg(type)
  
  if (!requireNamespace("Matrix", quietly = TRUE)) {
    stop("Package 'Matrix' must be installed to use predict.autotune_lasso().")
  }
  
  # ---- coefficients / nonzero types -----------------------------------------
  if (type == "coefficients") {
    if (is.null(path))
    {
      return(coef(object, intercept = TRUE, path = FALSE, sparse = TRUE))
    }
    if (path) 
    {
      return(coef(object, intercept = TRUE, path = TRUE, sparse = TRUE))
    }
  }
  
  # ---- response type --------------------------------------------------------
  # If no newx provided, return stored fitted values if you keep them
  if (is.null(newx)) {
    stop("newx is NULL and it needs new values for x at which predictions are to be made.")
  }
  
  newx <- as.matrix(newx)
  if (!is.null(object$nvars) && ncol(newx) != object$nvars) {
    stop("newx must have ", object$nvars, " columns.")
  }
  
  m <- nrow(newx)
  yhat <- newx %*% object$beta + rep(object$a0, m)
  return(yhat)
}

#' Plots for autotune_lasso objects
#'
#' @param x Fitted \code{"autotune_lasso"} object.
#' @param max_preds Integer input stating maximum number of predictors to include in the plot. 
#'   Default NULL (then max_preds is calculated internally). Plotting R-squared 
#'   involves running \code{max_preds} many linear models, in order to reduce 
#'   the computation, use it with the value \code{v} when you want to check 
#'   whether the dataset satisfies the sparsity assumption with 
#'   \eqn{\leq} \code{v} many predictors or not.
#' @param cumulative Logical input. If TRUE, plots cumulative R-squared; if FALSE, plots 
#'   adjusted R-squared. 
#' @param ... Other graphical parameters to plot
#'
#' @importFrom graphics abline grid legend lines par points text
#' @importFrom stats coef lm
#' @importFrom utils tail
#' @return Invisibly returns a data frame with columns:
#' \describe{
#'   \item{\code{n_predictors}}{Number of predictors included in the nested
#'   linear model.}
#'   \item{\code{predictor_index}}{Index of the predictor added at each step.}
#'   \item{\code{has_nonzero_beta}}{Logical value indicating whether the
#'   predictor added at each step has a non-zero coefficient.}
#'   \item{\code{r_squared}}{Cumulative R-squared value.}
#'   \item{\code{adj_r_squared}}{Adjusted R-squared value.}
#' }
#'
#' @examples
#' 
#' # Fit autotune lasso
#' set.seed(10)
#' n = 300
#' p = 500
#' s = 10
#' beta = c(rep(1, s), rep(0, p - s))
#' x = matrix(rnorm(n * p), ncol = p)
#' # Maunal sigma allocation
#' # y = x %*% beta + rnorm(n, sd = 1)
#' # Dynamic sigma allocation with snr specified
#' snr = 2
#' y = x %*% beta + rnorm(n, sd = sqrt(var(x%*%beta)/snr))
#' fit <- autotune_lasso(x, y)
#' 
#' # Basic diagnostic plot
#' plot(fit)
#' 
#' # Plot adjusted R-squared for first 15 predictors
#' plot(fit, max_preds = 15, cumulative = FALSE)
#' 
#' 
#' @seealso \code{\link{autotune_lasso}}
#'
#' @method plot autotune_lasso
#' @export
plot.autotune_lasso <- function(x,
                                max_preds = NULL,
                                cumulative = TRUE,...) {
  object <- x
  
  if (!inherits(object, "autotune_lasso")) {
    stop("Object must be of class 'autotune_lasso'")
  }

  dots <- list(...)
  cex.main <- if("cex.main" %in% names(dots)) dots$cex.main else 1.4
  cex.lab <- if("cex.lab" %in% names(dots)) dots$cex.lab else 1.6
  cex.axis <- if("cex.axis" %in% names(dots)) dots$cex.axis else 1.6
  dots$cex.main <- NULL
  dots$cex.lab <- NULL
  dots$cex.axis <- NULL

  sorted_predictors <- object$CD.path.details$sorted_predictors
  x <- object$x
  y <- object$y
  n_vars <- object$nvars
  n_obs <- object$nobs
  nnz_beta_count <- sum(object$beta != 0)

  if (is.null(x) || is.null(y)) {
    stop("Original data (x and y) not found in object. Please update your autotune_lasso function to store the original data.")
  }

  adj <- ifelse(n_obs > 10, 10, 1)
  if (is.null(max_preds)) {
    max_preds <- min(n_vars, n_obs - adj, as.integer(1.5 * nnz_beta_count))
  } else {
    max_preds <- min(max_preds, n_vars, n_obs - adj)
  }

  if (max_preds < 1) {
    stop("max_preds must be at least 1")
  }

  r_squared <- numeric(max_preds)
  adj_r_squared <- numeric(max_preds)

  message(sprintf(
    "Computing R-squared statistics for %d nested linear models.",
    max_preds
  ))

  for (i in 1:max_preds) {
    selected_vars <- sorted_predictors[1:i]

    if (is.matrix(x) || is.data.frame(x)) {
      x_subset <- x[, selected_vars, drop = FALSE]
    } else {
      stop("x must be a matrix or data frame")
    }

    fit <- lm(y ~ x_subset)
    df_used <- i + 1

    r_sq <- summary(fit)$r.squared
    r_squared[i] <- r_sq

    if (n_obs > df_used) {
      adj_r_squared[i] <- 1 - (1 - r_sq) * (n_obs - 1) / (n_obs - df_used)
    } else {
      adj_r_squared[i] <- NA
    }
  }

  predictor_indices <- sorted_predictors[1:max_preds]
  has_nonzero_beta <- object$beta[predictor_indices] != 0

  n_predictors <- 1:max_preds

  if (cumulative) {
    y_values <- r_squared
    y_label <- "Cumulative R-squared"
    main_title <- "R-squared vs Number of Predictors (Autotune Lasso Order)"
  } else {
    y_values <- adj_r_squared
    y_label <- "Adjusted R-squared"
    main_title <- "Adjusted R-squared vs Number of Predictors (Autotune Lasso Order)"
  }

 
  point_colors <- ifelse(has_nonzero_beta, "steelblue", "red")

  old_par <- par(mar = c(5.1, 4.5, 4.1, 2.1))  # Default is c(5.1, 4.1, 4.1, 2.1)
  on.exit(par(old_par))

  plot(n_predictors, y_values,
       type = "n",
       xlab = "Number of Predictors",
       ylab = y_label,
       # main = main_title,
       xlim = c(1, max_preds),
       ylim = c(min(y_values, na.rm = TRUE) * 0.95,
                max(y_values, na.rm = TRUE) * 1.05),
       cex.main = cex.main,
       cex.lab = cex.lab,
       cex.axis = cex.axis,
       ...)

  grid(col = "lightgray", lty = "dotted")

  for (i in 1:(max_preds - 1)) {
    if (!is.na(y_values[i]) && !is.na(y_values[i + 1])) {
      lines(c(i, i + 1), c(y_values[i], y_values[i + 1]),
            col = point_colors[i], lwd = 2)
    }
  }

  # Add points on top
  points(n_predictors, y_values, pch = 16, col = point_colors, cex = 1.2)

  # Add a reference line at R² = 0 if relevant
  if (min(y_values, na.rm = TRUE) < 0.1) {
    abline(h = 0, col = "gray", lty = 3)
  }


  legend("bottomright",
         legend = c("Non-zero coefficients", "Zero coefficients"),
         col = c("steelblue", "red"),
         pch = 16,
         bg = "white",
         cex = 1.5)


  final_r2 <- tail(y_values[!is.na(y_values)], 1)
  text(max_preds * 0.8, final_r2,
       paste("Final:", round(final_r2, 3)),
       pos = 3, col = "darkblue", cex = 0.9)

  invisible(data.frame(
    n_predictors = n_predictors,
    predictor_index = predictor_indices,
    has_nonzero_beta = has_nonzero_beta,
    r_squared = r_squared,
    adj_r_squared = adj_r_squared
  ))
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

autotune_lasso_cpp <- function(xin, yin, alpha = 0.01, standardize = TRUE, standardize_response = TRUE, intercept = TRUE, active = FALSE, trace_it = FALSE, tolerance = 1e-4, beta_tolerance = 1e-3, iter_max = 30L, beta_iter_max = 40L, active_iter_max = 5L, PR_norm_l2 = FALSE, tau = 0.5) {
    .Call(`_autotune_autotune_lasso_cpp`, xin, yin, alpha, standardize, standardize_response, intercept, active, trace_it, tolerance, beta_tolerance, iter_max, beta_iter_max, active_iter_max, PR_norm_l2, tau)
}

---
title: "autotune Lasso"
output: rmarkdown::html_vignette
# output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{Autotune Lasso}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

The R package `autotune` implements the Lasso with data-driven tuning for linear models.

## Installation and Loading
### Installation
The development version of the `autotune` package can be installed from GitHub using
```{r install-github, eval = FALSE}
# install.packages("devtools")
# Ensure that you have the Rcpp package installed with version >=1.0.13
# devtools::install_github("Tathagata-S/autotune")
```
When installing from GitHub, in order to build the package from source, you need to have the appropriate R development tools installed ([Rtools](https://cran.r-project.org/bin/windows/Rtools/) on Windows, or [these tools](https://mac.r-project.org/tools/) on Mac).


### Load Package
After installation, the package can be loaded in the standard way:
```{r setup}
library(autotune)
```

## High Dimensional Regression
`autotune` performs lasso via the `autotune_lasso()` function. 

We illustrate autotune lasso on simulated data using a linear model with $s=10$, $n=300$, $p=500$. Reader can also specify different configurations of data generating process in the following code chunk.

```{r}
set.seed(10)
n = 300
p = 500
s = 10
beta = c(rep(1, s), rep(0, p - s))
x = matrix(rnorm(n * p), ncol = p)
# Maunal sigma allocation
# y = x %*% beta + rnorm(n, sd = 1)
# Dynamic sigma allocation with snr specified
snr = 2
y = x %*% beta + rnorm(n, sd = sqrt(var(x%*%beta)/snr))
```


### Runtime of autotune Lasso
Given data (x, y), run the autotune lasso as follows with default $\alpha = 0.01$.

```{r}
ptm <- proc.time()
fit.autotune <- autotune_lasso(x, y, alpha = 0.01)
proc.time() - ptm
```

The regression coefficients $\beta$s, intercept, final lambda and the sequence of estimated sigmas can be extracted from the fitted autotune lasso object as follows
```{r}
b.autotune <- fit.autotune$beta
intercept.autotune <- fit.autotune$a0
lambda.autotune <- fit.autotune$lambda
sigma.seq.autotune <- fit.autotune$CD.path.details$sigma_sq_seq
sigma.estimate.autotune <- fit.autotune$sigma_sq
```


Now, `autotune` also supports the S3 function `coef` which returns the estimated regression coefficients $\beta$s along with the intercept

```{r}
head(coef(fit.autotune), n = 21)
```



## Comparison with Lasso tuned via Cross Validation (CV Lasso)

We contrast our solution with Cross-Validation-tuned Lasso using `cv.glmnet()` in the `glmnet` package.
```{r}
library(glmnet)
```


### Runtime of CV Lasso
```{r}
ptm2 <- proc.time()
fit.glmnet <- cv.glmnet(x, y)
proc.time()-ptm2
```
So, autotune Lasso shows faster runtimes as compared to CV Lasso.

## Comparison of quality of tuning between autotune and CV Lasso
Now, we will visualize the quality of lambdas selected by autotune and CV with respect to the 10-fold CV MSE and true Relative MSE.
```{r, fig.width=7.5, fig.height=7.5}
plot(fit.glmnet, cex.lab = 1.5, ylab = "10-fold CV MSE")

# plotting log of lambdas selected by CV
abline(v = log(c(fit.glmnet$lambda.min, fit.glmnet$lambda.1se)), lty = "dashed",col = c(rgb(0,1,0), rgb(0,0.75,0)), lwd = 3)

# plotting log of lambda selected by autotune
abline(v = log(lambda.autotune), col = "blue", lty = "dashed", lwd = 3)
legend("topleft", inset = 0.05,             legend = c("Autotune LASSO",
             "CV_LASSO(min)",
             "CV_LASSO(1se)"),
  col = c("blue", rgb(0,1,0), rgb(0,0.75,0)),
  lty = "dashed", lwd = 2, cex = 1)
```

```{r, fig.width=7.5, fig.height=7.5}
mse_glmnet <- apply(fit.glmnet$glmnet.fit$beta, 2, function(x) mean((x- beta)^2)/mean(beta^2))
ymin = 0.95 * min(mse_glmnet)
ymax = max(1, max(mse_glmnet))
oldpar <- par(mgp = c(3.5, 1, 0), mar = c(5, 4, 2.5, 0.5) + 1.4)
plot(log(fit.glmnet$lambda), mse_glmnet, 
     type = 'b', col = "red", cex.lab = 2, cex=2, cex.axis = 2,
     ylim = c(ymin, ymax),
     ylab = "Relative MSE w.r.t. true coefficients", xlab = expression(paste("log(",lambda,")")))

# plotting log of lambdas selected by CV
abline(v = log(c(fit.glmnet$lambda.min, fit.glmnet$lambda.1se)), lty = "dashed",col = c(rgb(0,1,0), rgb(0,0.75,0)), lwd = 3)

# plotting log of lambdas selected by autotune
abline(v = log(lambda.autotune), col = "blue", lty = "dashed", lwd = 3)
legend(
  "topleft", inset = 0.05,                  
  legend = c("Autotune LASSO",
             "CV_LASSO(min)",
             "CV_LASSO(1se)"),
  col = c("blue", rgb(0,1,0), rgb(0,0.75,0)),
  lty = "dashed", lwd = 2, cex = 1)
par(oldpar)
```


Across the lambda grid, we plot RMSE of solution path taken by different tuners.

```{r, fig.width=7.5, fig.height=7.5}
temp <- fit.autotune$CD.path.details$lambda0
seq.lambdas.autotune <- temp * c(var(y), sigma.seq.autotune)

final.rmse.autotune <- (mean(b.autotune - beta)^2) / (mean(beta)^2)
intermediate.rmse.autotune <- approx(x = fit.glmnet$lambda, y = mse_glmnet, xout = seq.lambdas.autotune)
intermediate.rmse.autotune$y[1] <- 1
rmse.path.autotune <- c(intermediate.rmse.autotune$y, final.rmse.autotune)

oldpar <- par(mgp = c(3.5, 1, 0), mar = c(5, 4, 2.5, 0.5) + 1.4)
plot(log(fit.glmnet$lambda), mse_glmnet, 
     type = 'b', col = "red", cex.lab = 2, cex=2, cex.axis = 2,
     ylim = c(min(ymin, 0.95 * final.rmse.autotune), ymax),
     ylab = "Relative MSE w.r.t. the true coefficients", xlab = expression(paste("log(",lambda,")")))

lines(c(log(seq.lambdas.autotune), log(fit.autotune$lambda)), 
      rmse.path.autotune, 
      col = "blue", 
      lwd = 2)                  
points(log(seq.lambdas.autotune), 
       intermediate.rmse.autotune$y, 
       col = "blue", 
       pch = 1,               
       lwd = 2,
       cex = 2) 
points(log(fit.autotune$lambda),
      final.rmse.autotune,
      col = "blue",
      pch = 11,
      lwd = 3,
      cex = 3
    )
par(oldpar)
```

```{r}
library(AUC)
```

```{r}
b.glmnet = coef(fit.glmnet, s = "lambda.min")[-1]
```


```{r}
auc(roc(abs(b.autotune), as.factor(beta != 0)))
auc(roc(abs(b.glmnet), as.factor(beta != 0)))
```

## Comparison on real data provided in Scaled Lasso's R package `scalreg`

The `scalreg` package was removed from CRAN in July 2026.

### Comparing prediction errors of autotune and benchmarks: CV and Scaled Lasso
```{r}
data("sp500")
attach(sp500)

scaled_lasso_benchmark <- read.csv(
  system.file(
    "extdata",
    "sp500-scalreg-1.0.1.csv",
    package = "autotune"
  )
)
scaled_lasso_observed <- scaled_lasso_benchmark[
  which(!scaled_lasso_benchmark$bootstrap),
  ,
  drop = FALSE
]

n <- 190
rang <- 252 - n

X_train = sp500.percent[1:n, 3:(dim(sp500.percent)[2])]
Y_train = sp500.percent[1:n, 1]

x_train = scale(X_train)
y_train = Y_train - mean(Y_train)

X_test = sp500.percent[(1:rang) + n, 3: (dim(sp500.percent)[2])]
Y_test = sp500.percent[(1:rang) + n, 1]
x_test <- scale(X_test)
y_test <- Y_test - mean(Y_test)
# Default value of beta_iter_max is 40
ans_autotune <- autotune_lasso(x_train, y_train, beta_iter_max = 40, trace_it = TRUE)
pred_err_autotune <- mean( (y_test - x_test %*% ans_autotune$beta)^2 )

pred_err_scallas <- scaled_lasso_observed$test_mse

cv_fit <- cv.glmnet(x_train, y_train, alpha = 1, intercept = F)
pred_err_cv_min <- mean( (y_test - predict(cv_fit, newx = x_test, s = "lambda.min"))^2 )
pred_err_cv_1se <- mean( (y_test - predict(cv_fit, newx = x_test, s = "lambda.1se"))^2 )


pred_err_autotune
pred_err_scallas
pred_err_cv_min
pred_err_cv_1se

scaled_lasso_observed$nonzero_coefs
sum(ans_autotune$beta != 0)
sum(coef(cv_fit, s = "lambda.min") != 0)
sum(coef(cv_fit, s = "lambda.1se") != 0)
detach(sp500)
```


### Comparing via bootstrap
```{r}
real_life_prediction_comparison <- function(x, y, x_test, y_test, scaled_lasso_result, alpha = 0.01, beta_iter_max = 40, plot_regu = TRUE, trace_it = TRUE){

ans_autotune <- autotune_lasso(x, y, alpha = alpha, beta_iter_max = beta_iter_max, trace_it = trace_it)
# ans_glmnet <- glmnet(x, y, alpha = 1, intercept = FALSE)
cv_fit <- cv.glmnet(x, y, alpha = 1, intercept = F)
vary <- var(y_test)
mse_glmnet <- apply(cv_fit$glmnet.fit$beta, 2, function(b) mean((y_test - x_test%*%b)^2)/vary)

# log_glmnet_lambdas <- log(fit.glmnet$lambda)
# log_autotune_lambdas <- log(intermediate_lambdas_used)



if(plot_regu) {
  intermediate_lambdas_used <- c( c(var(y)/2, ans_autotune$CD.path.details$sigma_sq_seq) * ans_autotune$CD.path.details$lambda0)
  
  autotune_lambdas <- c(intermediate_lambdas_used, ans_autotune$lambda)
  
  final.rmse.autotune <- mean((y_test - x_test%*%ans_autotune$beta)^2) / vary
  intermediate.rmse.autotune <- approx(x = log(cv_fit$lambda), y = mse_glmnet, xout = log(intermediate_lambdas_used), rule = 2)
  intermediate.rmse.autotune$y[1] <- 1
  rmse.path.autotune <- c(intermediate.rmse.autotune$y, final.rmse.autotune)
  
  
  ysquaredmean <- mean(y^2)
  cv_fit$cvm <- cv_fit$cvm/ysquaredmean
  cv_fit$cvsd <- cv_fit$cvsd/ysquaredmean
  cv_fit$cvup <- cv_fit$cvm + cv_fit$cvsd
  cv_fit$cvlo <- cv_fit$cvm - cv_fit$cvsd
  
  cv_fit_lowerlim <- min(cv_fit$cvm - cv_fit$cvsd, mse_glmnet)
  cv_fit_upperlim <- max(cv_fit$cvm + cv_fit$cvsd, mse_glmnet)
  
  ymin = 0.95 * min(cv_fit_lowerlim , min(rmse.path.autotune))
  ymax = max(cv_fit_upperlim, rmse.path.autotune)
  ymax_trunc = ymin + 0.8 * (ymax - ymin)  # Truncate vertical lines here
  
  ydiff = ymax - ymin
  oldpar <- par(mgp = c(3.5, 1, 0), mar = c(6, 5, 4, 2) + 0.1)
  on.exit(par(oldpar), add = TRUE)
  
  par(mar = c(5, 7, 4, 2))
  plot(cv_fit,
       ylim = c(ymin, max(cv_fit_upperlim, rmse.path.autotune)),
       cex.lab = 2,
       cex.axis = 2,
       ylab = expression(
    "||" * (Y[test] - X[test] * hat(beta)) * "||"[2]^2
    ~ "/" ~
    "||" * Y[test]  * "||"[2]^2
  ))
  # expression(paste(sqrt("RMSE")))
  
  points(log(cv_fit$lambda), mse_glmnet,
         type = 'b', col = "orange", lwd = 3)
  
  
  lines(rep(log(ans_autotune$lambda), 2),
        c(ymin, ymin + 0.92 * ydiff), col = "blue", lwd = 0.5, lty = "dashed")
  
  lines(log(autotune_lambdas), rmse.path.autotune,
        col = "blue", lwd = 1.5)
  
  points(log(autotune_lambdas), rmse.path.autotune,
         col = "blue", pch = 1, lwd = 1.5, cex = 1)
  
  points(log(ans_autotune$lambda), final.rmse.autotune,
         col = "blue", pch = 11, lwd = 2, cex = 2)
  
  
  
  vlines <- c(
    log(cv_fit$lambda.min),
    log(cv_fit$lambda.1se)
  )
  vline_labels <- c("CV(min)", "CV(1se)")
  vline_colors <- c(rgb(0,1,0), rgb(0,0.75,0))
  vline_ltys <- c("dashed", "dashed")
  vline_lwds <- c(3, 3)
  
  for (i in seq_along(vlines)) {
    lines(rep(vlines[i], 2),
          c(ymin, ymin + (0.55 + i * 0.1) * ydiff),
          col = vline_colors[i], lty = vline_ltys[i],
          lwd = vline_lwds[i])
  
    text(x = vlines[i], y = ymin + (0.57 + i * 0.1) * ydiff,
         labels = vline_labels[i], srt = -30, adj = 0,
         xpd = TRUE, cex = 1.4, col = vline_colors[i])
  }
  
  text(x = log(ans_autotune$lambda),
       y =  ymin + 0.95 * ydiff,
       labels = "Autotune", srt = -30, adj = 0,
       xpd = TRUE, cex = 1.4, col = "blue")
  
  if (!is.null(cv_fit$cvsd)) {
    arrows(x0 = log(cv_fit$lambda), 
           y0 = cv_fit$cvm - cv_fit$cvsd,
           x1 = log(cv_fit$lambda),
           y1 = cv_fit$cvm + cv_fit$cvsd,
           angle = 90, code = 3, length = 0.05, col = "red", lwd = 1.5)
  }
  
  
  legend("topright",
         inset = c(0.05, 0.02),
         legend = c("CV Error", "Test Error"),
         col = c("red", "orange"),
         pt.cex = c(1.5, 2), pch = c(19, 19),
         lwd = c(1, 3),
         text.col = c("red", "orange"),
         horiz = FALSE,
         bty = "n")

}

# mse_autotune <- mean( (y_test - x_test %*% ans_autotune$beta)^2 )
# mse_cv_min <- mean( (y_test - predict(cv_fit, newx = x_test, s = "lambda.min"))^2 )
# mse_cv_1se <- mean( (y_test - predict(cv_fit, newx = x_test, s = "lambda.1se"))^2 )


oos_scallas <- scaled_lasso_result$out_of_sample_r2
oos_autotune <- 1 -  mean( (y_test - x_test %*% ans_autotune$beta)^2 ) / vary
oos_cv_min <- 1 - mean( (y_test - predict(cv_fit, newx = x_test, s = "lambda.min"))^2 ) / vary
oos_cv_1se <- 1 - mean( (y_test - predict(cv_fit, newx = x_test, s = "lambda.1se"))^2 ) / vary

return(data.frame(out_of_sample_autotune = oos_autotune,
out_of_sample_scaled_lasso = oos_scallas,
out_of_sample_cv_min = oos_cv_min,
out_of_sample_cv_1se = oos_cv_1se,
nonzero_coefs_autotune = sum(ans_autotune$beta != 0),
nonzero_coefs_scaled_lasso = scaled_lasso_result$nonzero_coefs,
nonzero_coefs_cvmin = sum(coef(cv_fit, s = "lambda.min") != 0),
nonzero_coefs_cv1se = sum(coef(cv_fit, s = "lambda.1se") != 0)))
}
```


```{r, fig.width=7.5, fig.height=7.5}
ans <- real_life_prediction_comparison(
  x_train,
  y_train,
  x_test,
  y_test,
  scaled_lasso_result = scaled_lasso_observed,
  plot_regu = TRUE
)
names(ans)
```

```{r}
library(dplyr)
```



```{r, fig.width=7.5, fig.height=7.5}
#Number of bootstraps
B <- 20


result_list <- list()

result_list[[1]] <- data.frame(
  n = n,
  bootstrap = FALSE,
  real_life_prediction_comparison(
    x_train,
    y_train,
    x_test,
    y_test,
    scaled_lasso_result = scaled_lasso_observed,
    plot_regu = FALSE,
    trace_it = FALSE
  )[1:4]
)

for( j in (1:B)+1) {
  set.seed(j)
  boot.sample <- sample(n, size = n, replace = TRUE)
  boot.x <- x_train[boot.sample, ]
  boot.y <- y_train[boot.sample]
  scaled_lasso_result <- scaled_lasso_benchmark[
    which(scaled_lasso_benchmark$seed == j),
    ,
    drop = FALSE
  ]
  result_list[[j]] <- c(
    n = n,
    bootstrap = TRUE,
    real_life_prediction_comparison(
      boot.x,
      boot.y,
      x_test,
      y_test,
      scaled_lasso_result = scaled_lasso_result,
      plot_regu = FALSE,
      trace_it = FALSE
    )[1:4]
  )
}

mat <- do.call(rbind, result_list)     
df <- as.data.frame(mat, stringsAsFactors = FALSE)

colnames(df) <- c("n", "bootstrap", "Autotune",
                  "Scaled", "CVmin",
                  "CV1se")
df <- df %>%
  mutate(
    n = as.integer(n),
    bootstrap = as.logical(bootstrap), 
    across(!c(n, bootstrap), as.numeric)
  )
```



```{r, fig.width=7.5, fig.height=7.5}
alg_cols <- colnames(df)[3:6]
boot_only <- df %>% filter(bootstrap == TRUE)
boxlist <- lapply(alg_cols, function(cn) boot_only[[cn]])
names(boxlist) <- alg_cols

# Uncomment the following line and line 448 for saving the boxplot in your working directory

# pdf(file.path(paste0("OOS_boxplot_for_n=",df[1,1],"_B=", B,".pdf")), width = 8, height = 9)

oldpar <- par(mar = c(4, 6, 4, 2))
boxplot(boxlist, main = paste0("Bootstrapped Out-of-Sample R^2 for n = ", df[1,1]), ylab = expression(paste("Out-of-Sample ",R^2)), ylim = c(max(0.98 * min(df[alg_cols]), 0), 1),
        cex.lab = 2,     
  cex.axis = 1.8,    
  cex.main = 1.5)

nonboot <- df %>% filter(bootstrap == FALSE)
if(nrow(nonboot) > 0){
  points(1:4, as.numeric(nonboot[1, alg_cols]), pch = 18, col = "red", cex = 2)
}

legend(
  "topright",
  legend = expression(paste("Observed O.O.S. ",R^2)),
  col = "red",
  pch = 18,
  pt.cex = 2,
  cex = 2,          
  bty = "n"           
)

par(oldpar)
# dev.off()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autotune_lasso.R
\name{autotune_lasso}
\alias{autotune_lasso}
\title{Data-adaptive automatic and fast tuning of LM with Lasso regularization}
\usage{
autotune_lasso(
  x,
  y,
  alpha = 0.01,
  standardize = TRUE,
  standardize_response = TRUE,
  intercept = TRUE,
  active = FALSE,
  trace_it = FALSE,
  tolerance = 1e-04,
  beta_tolerance = 0.001,
  iter_max = 30,
  beta_iter_max = 40,
  active_iter_max = 5,
  PR_norm_l2 = FALSE,
  ...
)
}
\arguments{
\item{x}{Matrix of predictors, with one column for each predictor.
Dimension will be \code{nobs} \eqn{\times} \code{nvars}; so each row is a new observation
and \code{nvars} > 1.}

\item{y}{Vector of responses.}

\item{alpha}{Default 0.01, significance level of sequential F-tests 
used for estimation of support set.}

\item{standardize}{Logical flag for standardization of all variables in x, prior to
fitting the model sequence. The coefficients are always returned on the
original scale. Default value is \code{TRUE}.}

\item{standardize_response}{Logical flag for demeaning the reponse variable y.
Default value is \code{TRUE}.}

\item{intercept}{Should intercept(s) be fitted (default=\code{TRUE}) or set to zero
(\code{FALSE}).}

\item{active}{Should active set selection be used (\code{TRUE}) or avoided
(default = \code{FALSE}). It is under experimentation phase, use it 
with care.}

\item{trace_it}{Logical input, default \code{FALSE}; if \code{TRUE}
prints out the iteration number while running, useful for big datasets that 
take a long time to fit.}

\item{tolerance}{Numeric input for an additional stopping criteria on the
coordinate descent when sigma is updating. Stops that coordinate
descent when the relative change between successive iterates of
coefficients' estimates is less than the \code{tolerance}. Default value 
is \eqn{10^{-4}}.}

\item{beta_tolerance}{Numeric input for the stopping criteria on the
coordinate descent when sigma is not updating. Stops that coordinate
descent when the relative change between successive iterates of
coefficients' estimates is less than the \code{beta_tolerance}.
Default value is \eqn{10^{-3}}.}

\item{iter_max}{Maximum number of iterations of coordinate descent
allowed when sigma is updating. Default value is 30.}

\item{beta_iter_max}{Maximum number of iterations of coordinate 
descent allowed when sigma is not updating. Default value is 40.}

\item{active_iter_max}{If \code{active = TRUE}, maximum number of 
times the active set is updated as per the violations of KKT 
conditions. Default value is 5.}

\item{PR_norm_l2}{Logical flag to whether use the l2 norm of partial 
residuals for ordering them instead of the default l1 norm.}

\item{...}{Additional arguments passed to the internal fitting routine.}
}
\value{
A list with various intermediate and final outputs produced by autotune Lasso in its regularization path.

\item{beta}{ Final estimates of regression coefficients. }
\item{a0}{ Intercept of the fit. }
\item{lambda}{Final thresholding value \eqn{\lambda =
\lambda_0\hat\sigma^2} used in the coordinate descent after noise
variance estimate \eqn{\hat\sigma^2} has converged.}
\item{sigma_sq}{ Final estimate of noise variance \eqn{\sigma^2}}
\item{nobs}{ Number of observations.}
\item{nvars}{ Number of variables.}
\item{CD.path.details}{ A list of additional details about the coordinate descent path taken by
autotune Lasso:
  \describe{
    \item{\code{sorted_predictors}}{Decreasing ordering of predictors in terms of
    their contribution to predicting the response values.}
    \item{\code{sigma_sq_seq}}{A \code{no_of_iterations}-length sequence of
    noise variance estimates \eqn{\sigma^2}.}
    \item{\code{beta_matrix}}{A (\code{length(sigma_sq_seq) + 1})
    \eqn{\times} \code{nvars} matrix of estimated coefficients.}
    \item{\code{no_of_iter_before_lambda_conv}}{Number of coordinate descent
    iterations performed before the noise variance estimate
    \eqn{\hat{\sigma}^2} converged.}
    \item{\code{no_of_iter_after_lambda_conv}}{After the noise variance estimate
    \eqn{\hat{\sigma}^2} has converged, the number of coordinate descent
    iterations required for coefficients \eqn{\hat\beta} to converge.}
    \item{\code{no_of_iterations}}{Total number of coordinate descent iterations
    implemented by autotune Lasso.}
    \item{\code{lambda0}}{Value of \eqn{\lambda_0} used in autotune Lasso.
    Refer to the original paper for details.}
    \item{\code{support_set}}{Final set of predictors included in the support set
    for noise variance estimation by autotune Lasso.}
    \item{\code{count_sig_beta}}{A \code{no_of_iterations}-length vector containing
    the support set sizes across the coordinate descent iterations while the
    noise variance estimate is being updated.}
    \item{\code{null_support}}{Boolean output indicating whether the final support
    set estimate is a null set. If so, \code{autotune_lasso} uses the support set
    estimate from the previous iteration to obtain the final noise variance
    estimate \eqn{\sigma^2}.}
    \item{\code{active_iterations}}{If \code{active = TRUE}, the number of times
    the active set is updated according to violations of the KKT conditions.}
    \item{\code{active_set_sizes}}{An \code{active_iterations}-length vector
    containing the active-set sizes used for coordinate descent across the
    active-set iterations.}
  }
}
}
\description{
Fits a linear model via alternative optimization of penalized gaussian maximum likelihood 
which is a biconvex function of regression coefficients \eqn{\beta} and noise variance \eqn{\sigma^2}. 
The regularization path of \code{autotune_lasso} quickly picks out a good lambda for Lasso and then
returns the corresponding linear fit along with various attributes related to the fit.
}
\examples{
library(autotune)
set.seed(10)
n <- 80
p <- 400
s <- 5
snr <- 4
betatrue <- c(rep(1,s), rep(0, p - s))
x <- matrix(rnorm(n * p), ncol = p)
error.sd <- sqrt((betatrue \%*\% betatrue)/snr)
err <- rnorm(n, sd = error.sd)
y <- x \%*\% betatrue + err
ans <- autotune_lasso(x, y, trace_it = TRUE)
b <- betatrue
# The Predictors which are actually significant:
which(b != 0)
# The Predictors which had nonzero estmated coefficients:
which(ans$beta != 0)
# Top 10 predictors X_i's in the ranking of X_i's given by autotune:
ans$CD.path.details$sorted_predictors[1:10]
# Cardinality of autotune's estimated support set in each CD iteration before lambda converged:
ans$CD.path.details$count_sig_beta
# Sigma estimates in each CD iteration:
ans$CD.path.details$sigma_sq_seq
# Empirical noise variance:
var(err)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.autotune_lasso.R
\name{plot.autotune_lasso}
\alias{plot.autotune_lasso}
\title{Plots for autotune_lasso objects}
\usage{
\method{plot}{autotune_lasso}(x, max_preds = NULL, cumulative = TRUE, ...)
}
\arguments{
\item{x}{Fitted \code{"autotune_lasso"} object.}

\item{max_preds}{Integer input stating maximum number of predictors to include in the plot. 
Default NULL (then max_preds is calculated internally). Plotting R-squared 
involves running \code{max_preds} many linear models, in order to reduce 
the computation, use it with the value \code{v} when you want to check 
whether the dataset satisfies the sparsity assumption with 
\eqn{\leq} \code{v} many predictors or not.}

\item{cumulative}{Logical input. If TRUE, plots cumulative R-squared; if FALSE, plots 
adjusted R-squared.}

\item{...}{Other graphical parameters to plot}
}
\value{
Invisibly returns a data frame with columns:
\describe{
  \item{\code{n_predictors}}{Number of predictors included in the nested
  linear model.}
  \item{\code{predictor_index}}{Index of the predictor added at each step.}
  \item{\code{has_nonzero_beta}}{Logical value indicating whether the
  predictor added at each step has a non-zero coefficient.}
  \item{\code{r_squared}}{Cumulative R-squared value.}
  \item{\code{adj_r_squared}}{Adjusted R-squared value.}
}
}
\description{
Plots for autotune_lasso objects
}
\examples{

# Fit autotune lasso
set.seed(10)
n = 300
p = 500
s = 10
beta = c(rep(1, s), rep(0, p - s))
x = matrix(rnorm(n * p), ncol = p)
# Maunal sigma allocation
# y = x \%*\% beta + rnorm(n, sd = 1)
# Dynamic sigma allocation with snr specified
snr = 2
y = x \%*\% beta + rnorm(n, sd = sqrt(var(x\%*\%beta)/snr))
fit <- autotune_lasso(x, y)

# Basic diagnostic plot
plot(fit)

# Plot adjusted R-squared for first 15 predictors
plot(fit, max_preds = 15, cumulative = FALSE)


}
\seealso{
\code{\link{autotune_lasso}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coef.R, R/coef.autotune_lasso.R,
%   R/coef.autotune_lasso_path.R
\name{coef.autotune_lasso}
\alias{coef.autotune_lasso}
\alias{coef.autotune_lasso_path}
\title{Extract coefficients for autotune_lasso objects}
\usage{
\method{coef}{autotune_lasso}(object, intercept = TRUE, path = FALSE, sparse = TRUE, ...)

\method{coef}{autotune_lasso_path}(object, intercept = TRUE, a0 = NULL, ...)
}
\arguments{
\item{object}{An \code{"autotune_lasso"} object (for \code{coef.autotune_lasso})
or \code{"autotune_lasso_path"} object (for \code{coef.autotune_lasso_path}).}

\item{intercept}{Logical; include the intercept term? If \code{FALSE}, the intercept
row is omitted.}

\item{path}{Logical; if \code{TRUE}, return the entire coefficient path across
different lambdas.}

\item{sparse}{Logical; controls the output format of \code{coef.autotune_lasso()} when
\code{path = FALSE}. If \code{TRUE}, returns a sparse \code{dgCMatrix} (glmnet-style);
if \code{FALSE}, returns a base numeric vector.}

\item{...}{Other arguments to coef.}

\item{a0}{Intercept of the fitted autotune regression. Used only when \code{intercept = TRUE}.}
}
\value{
\itemize{
  \item \code{coef.autotune_lasso(path = FALSE, sparse = TRUE)} returns a sparse \code{dgCMatrix}
    of dimension (\code{nvars} + 1) \eqn{\times 1} (or \code{nvars}  \eqn{\times 1} if \code{intercept = FALSE}).
  \item \code{coef.autotune_lasso(path = FALSE, sparse = FALSE)} returns a named numeric vector.
  \item \code{coef.autotune_lasso(path = TRUE)} calls \code{coef.autotune_lasso_path()}, which returns a sparse
    \code{dgCMatrix} of dimension (\code{nvars}  + 1) \eqn{\times} (k + 1) (or \code{nvars}  \eqn{\times} (k + 1) if
    \code{intercept = FALSE}), with columns named \code{lambda_1, ..., lambda_k, final_lambda}.
}
}
\description{
This function extracts the regression coefficients estimated by an autotune
lasso model, using the stored \code{"autotune_lasso"} object
}
\examples{
set.seed(10)
n = 300
p = 500
s = 10
beta = c(rep(1, s), rep(0, p - s))
x = matrix(rnorm(n * p), ncol = p)
# Maunal sigma allocation
# y = x \%*\% beta + rnorm(n, sd = 1)
# Dynamic sigma allocation with snr specified
snr = 2
y = x \%*\% beta + rnorm(n, sd = sqrt(var(x\%*\%beta)/snr))
fit <- autotune_lasso(x, y)
coef(fit)                    # final solution
coef(fit, path = TRUE)        # coefficient path (sparse)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autotune-package.R
\docType{package}
\name{autotune-package}
\alias{autotune}
\alias{autotune-package}
\title{autotune: Lasso with data-driven tuning for linear models}
\description{
This package fits lasso path for high-dimensional regression using
coordinate descent while automatically tuning it's regularization parameter.
autotune Lasso is 10-50 times than the standard glmnet implementation of lasso,
and over 100 times faster than scaled lasso. Additionally, it gives reliable 
noise level estimate for the regression problem.
}
\details{
\tabular{ll}{ Package: \tab autotune\cr Type: \tab Package\cr Version: \tab
1.0\cr Date: \tab 2025-09-18\cr }
Very simple to use. Accepts \code{x,y} data for linear regression model,
and produces the regularization path which automatically estimates an
optimal tuning parameter \code{lambda}.\cr
}
\section{Functions}{
 Only 1 function, more autotune algorithms for VAR, random forests,
coming soon!
  \itemize{
    \item \code{\link{autotune_lasso}}
  }
}

\examples{

set.seed(10)
n <- 80
p <- 500
s <- 5
type <- 1
snr <- 3
betatrue <- c(rep(1,s), rep(0, p - s))
x <- scale(matrix(rnorm(n * p), ncol = p))
error.sd <- sqrt((betatrue \%*\% betatrue)/snr)
err <- rnorm(n, sd = error.sd)
y <- x \%*\% betatrue + err
y <- y - mean(y)
ans <- autotune_lasso(x, y, trace_it = TRUE)
b <- betatrue
# The Predictors which are actually significant:
which(b != 0)
# The Predictors which had nonzero estmated coefficients:
which(ans$beta != 0)
# Top 10 predictors X_i's in the ranking of X_i's given by autotune:
ans$sorted_predictors[1:10] + 1
# No of significant predictors in each CD iteration when sigma_hat is allowed to vary:
ans$count_sig_beta
# Sigma estimates in each CD iteration:
ans$sigma2_seq
# Empirical noise variance:
var(err)


}
\author{
Tathagata Sadhukhan Ines Wilms Stephan Smeekes
and Sumanta Basu\cr Maintainer: Tathagata Sadhukhan(ts767@cornell.edu)
}
\keyword{accurate}
\keyword{data-driven-tuning}
\keyword{fast}
\keyword{lasso}
\keyword{package}

\name{sp500}
\alias{sp500}
\docType{data}
\title{S&P 500 Stock Data}
\description{
The \code{sp500} data file contains a year's worth of close-of-day data
for most of the Standard and Poor's 500 stocks. The data are in reverse
chronological order, with the first row corresponding to December 31,
2008.
}
\usage{
data("sp500")
}
\format{
A list containing the following two matrices:
\describe{
  \item{\code{sp500.percent}}{
    A 252 by 494 matrix of daily percentage changes. The first
    column is the DJIA index, the second is the S&P 500 index, and the
    remaining columns are labeled individual stocks.
  }
  \item{\code{sp500.2008}}{
    A 253 by 494 matrix of raw close-of-day data. The first
    column is the DJIA index, the second is the S&P 500 index, and the
    remaining columns are labeled individual stocks.
  }
}
}
\source{
Redistributed from version 1.0.1 of the GPL-2-licensed R package
\pkg{scalreg}.
}
\references{
This database was used in the R package \pkg{plus}.
}
\examples{
data("sp500")
names(sp500)
dim(sp500$sp500.percent)

attach(sp500)
head(sp500.percent[, 1:5])
detach(sp500)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.autotune_lasso.R
\name{predict.autotune_lasso}
\alias{predict.autotune_lasso}
\title{Predict from a autotune_lasso fit}
\usage{
\method{predict}{autotune_lasso}(
  object,
  newx = NULL,
  type = c("response", "coefficients"),
  path = NULL,
  ...
)
}
\arguments{
\item{object}{An object of class \code{autotune_lasso}.}

\item{newx}{Numeric matrix of new \code{x} (rows = observations,
columns = predictors) at which predictions are required. This 
argument is not used for \code{type="coefficients")}.}

\item{type}{Character; one of \code{"response"}, \code{"coefficients"}.}

\item{path}{For \code{type = "coefficients"}, \code{NULL} returns the final
coefficients and \code{TRUE} returns the full coefficient path. This
argument is ignored for \code{type = "response"}.}

\item{...}{Further arguments passed from the generic.}
}
\value{
If \code{type = "response"}, returns the prediction at the x provided for prediction.

If \code{type = "coefficients"}, returns the fitted coefficients.
}
\description{
Predict from a autotune_lasso fit
}
\examples{
set.seed(10)
n = 300
p = 500
s = 10
beta = c(rep(1, s), rep(0, p - s))
x = matrix(rnorm(n * p), ncol = p)
# Maunal sigma allocation
# y = x \%*\% beta + rnorm(n, sd = 1)
# Dynamic sigma allocation with snr specified
snr = 2
y = x \%*\% beta + rnorm(n, sd = sqrt(var(x\%*\%beta)/snr))
fit <- autotune_lasso(x, y)
predict(fit, newx = x[1:5, , drop = FALSE])
predict(fit, type = "coefficients")

predict(fit, type = "coefficients", path = TRUE)

}
